@@ -27,11 +27,10 @@
 //! ------------
 
 
-//extern crate ndarray;
-//extern crate ndarray_linalg;
-
 pub mod tensor;
 pub mod quaternion;
 pub mod typed_tensor;
 pub mod tensor_trait;
 pub mod tensor_impl;
+#[cfg(feature = "use-serde")]
+pub mod serde;
