@@ -2,7 +2,6 @@
 //! A general tensor type.
 //! 
 
-// extern crate ndarray;
 // Default value type is f32.
 // Right dimension of the tensor changes fastest.
 use std::rc::Rc;
@@ -16,8 +15,6 @@ use serde::{Serialize, Deserialize};
 use std::fmt;
 
 
-
-
 use super::typed_tensor::TypedTensor;
 use crate::tensor_impl::gen_tensor::GenTensor;
 
@@ -737,6 +734,15 @@ impl Tensor {
          Tensor { v: Rc::new(RefCell::new(r2))},
         )
     }
+
+    pub fn inner(&self) -> Rc<RefCell<TypedTensor>> {
+	self.v.clone()
+    }
+    pub fn set_inner(tt: TypedTensor) -> Tensor {
+	Tensor {
+	    v: Rc::new(RefCell::new(tt))
+	}
+    }
 }
 
 impl fmt::Display for Tensor {
@@ -780,6 +786,8 @@ impl Clone for Tensor {
 //    }
 //}
 
+
+#[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 #[derive(Clone, Copy, PartialEq)]
 pub enum PaddingMode{
     Zeros,
