@@ -14,12 +14,21 @@ use super::tensor_trait::compare_tensor::CompareTensor;
 use super::tensor_trait::elemwise::ElemwiseTensorOp;
 use super::tensor_trait::index_slicing::IndexSlicing;
 use super::tensor_trait::convolution::{Convolution};
-#[cfg(feature = "use-blas")]
-use super::tensor_impl::lapack_tensor::convolution::{gemm_conv_f32, gemm_conv_f64};
 use super::tensor_trait::reduction::ReduceTensor;
 use super::tensor_trait::linalg::LinearAlgbra;
 use super::tensor_trait::rand::Random;
 
+#[cfg(feature = "use-blas-lapack")]
+use super::tensor_impl::lapack_tensor::convolution::{gemm_conv_f32, gemm_conv_f64};
+#[cfg(feature = "use-blas-lapack")]
+use crate::tensor_impl::lapack_tensor::elemwise::{add_f32, add_f64,
+                                                  sub_f32, sub_f64,};
+#[cfg(feature = "use-blas-lapack")]
+use crate::tensor_impl::lapack_tensor::{matmul_f64, matmul_f32};
+#[cfg(feature = "use-blas-lapack")]
+use crate::tensor_impl::lapack_tensor::linalg::{svd_f64, svd_f32};
+
+
 #[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub enum TypedTensor {
     Typef32(GenTensor<f32>),
@@ -126,31 +135,31 @@ macro_rules! typed_tensor_method_single_same_option_2_return {
     }
 }
 
-macro_rules! typed_tensor_method_single_same_option_3_return {
-    ($a:ident) => {
-        pub fn $a(&self) -> Option<[TypedTensor; 3]> {
-            match &self {
-                TypedTensor::Typef32(v1) => {
-                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Typef32(r1),
-                                                TypedTensor::Typef32(r2),
-                                                TypedTensor::Typef32(r3),])
-                },
-                TypedTensor::Typef64(v1) => {
-                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Typef64(r1),
-                                                TypedTensor::Typef64(r2),
-                                                TypedTensor::Typef64(r3),])
-                },
-                #[cfg(feature = "use-cuda")]
-                TypedTensor::Cudaf32(v1) => {
-                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Cudaf32(r1),
-                                                TypedTensor::Cudaf32(r2),
-                                                TypedTensor::Cudaf32(r3),])
-                },
-                //_ => {panic!("should have same tensor type!");},
-            }
-        }
-    }
-}
+//macro_rules! typed_tensor_method_single_same_option_3_return {
+//    ($a:ident) => {
+//        pub fn $a(&self) -> Option<[TypedTensor; 3]> {
+//            match &self {
+//                TypedTensor::Typef32(v1) => {
+//                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Typef32(r1),
+//                                                TypedTensor::Typef32(r2),
+//                                                TypedTensor::Typef32(r3),])
+//                },
+//                TypedTensor::Typef64(v1) => {
+//                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Typef64(r1),
+//                                                TypedTensor::Typef64(r2),
+//                                                TypedTensor::Typef64(r3),])
+//                },
+//                #[cfg(feature = "use-cuda")]
+//                TypedTensor::Cudaf32(v1) => {
+//                    v1.$a().map(|[r1, r2, r3]| [TypedTensor::Cudaf32(r1),
+//                                                TypedTensor::Cudaf32(r2),
+//                                                TypedTensor::Cudaf32(r3),])
+//                },
+//                //_ => {panic!("should have same tensor type!");},
+//            }
+//        }
+//    }
+//}
 
 
 
@@ -162,7 +171,7 @@ impl Default for TypedTensor {
 
 impl TypedTensor {
     pub fn new() -> TypedTensor {
-        // Default value type is f32.
+        // Default value type is f64.
         TypedTensor::Typef64(GenTensor::new())
     }
 
@@ -313,12 +322,12 @@ impl TypedTensor {
         match (self, o) {
             (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
                 let mut v = v1.clone();
-                v.set_patch(&v2, range, step);
+                v.set_patch(v2, range, step);
                 TypedTensor::Typef32(v)
             },
             (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {
                 let mut v = v1.clone();
-                v.set_patch(&v2, range, step);
+                v.set_patch(v2, range, step);
                 TypedTensor::Typef64(v)
             },
             _ => {panic!("should have same tensor type!");},
@@ -639,13 +648,73 @@ impl TypedTensor {
     // assert_eq!(m3.get(&vec![0,0]), 2.);
     // assert_eq!(m3.get(&vec![1,1]), 8.);
     // ```
-    typed_tensor_method!(add);
-    typed_tensor_method!(sub);
+    pub fn add(&self, o: &TypedTensor) -> TypedTensor {
+        match (&self, o) {
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {TypedTensor::Typef32(v1.add(v2))},
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {TypedTensor::Typef64(v1.add(v2))},
+              
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
+                TypedTensor::Typef32(add_f32(v1, v2))
+            },
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {
+                TypedTensor::Typef64(add_f64(v1, v2))
+            },
+              
+            #[cfg(feature = "use-cuda")]
+            (TypedTensor::Cudaf32(v1), TypedTensor::Cudaf32(v2)) => {TypedTensor::Cudaf32(v1.add(v2))},
+            _ => {panic!("should have same tensor type!");},
+        }
+    }
+    pub fn sub(&self, o: &TypedTensor) -> TypedTensor {
+        match (&self, o) {
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {TypedTensor::Typef32(v1.sub(v2))},
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {TypedTensor::Typef64(v1.sub(v2))},
+              
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
+                TypedTensor::Typef32(sub_f32(v1, v2))
+            },
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {
+                TypedTensor::Typef64(sub_f64(v1, v2))
+            },
+              
+            #[cfg(feature = "use-cuda")]
+            (TypedTensor::Cudaf32(v1), TypedTensor::Cudaf32(v2)) => {TypedTensor::Cudaf32(v1.sub(v2))},
+            _ => {panic!("should have same tensor type!");},
+        }
+    }
     typed_tensor_method!(mul);
     typed_tensor_method!(div);
-
+    pub fn matmul(&self, o: &TypedTensor) -> TypedTensor {
+        match (&self, o) {
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {TypedTensor::Typef32(v1.matmul(v2))},
+            #[cfg(not(feature = "use-blas-lapack"))]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {TypedTensor::Typef64(v1.matmul(v2))},
+              
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
+                TypedTensor::Typef32(matmul_f32(v1, v2))
+            },
+            #[cfg(feature = "use-blas-lapack")]
+            (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {
+                TypedTensor::Typef64(matmul_f64(v1, v2))
+            },
+              
+            #[cfg(feature = "use-cuda")]
+            (TypedTensor::Cudaf32(v1), TypedTensor::Cudaf32(v2)) => {TypedTensor::Cudaf32(v1.matmul(v2))},
+            _ => {panic!("should have same tensor type!");},
+        }
+    }
     typed_tensor_method!(mm);
-    typed_tensor_method!(matmul);
+    
     pub fn outer(&self, o: &TypedTensor, avg: Option<bool>) -> TypedTensor {
         match (&self, o) {
             (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
@@ -747,7 +816,43 @@ impl TypedTensor {
     typed_tensor_method_single_same_option_2_return!(eigen);
     typed_tensor_method_single_same_option_1_return!(cholesky);
     typed_tensor_method_single_same_option_1_return!(det);
-    typed_tensor_method_single_same_option_3_return!(svd);
+    pub fn svd(&self) -> Option<[TypedTensor; 3]> {
+        match &self {
+	    #[cfg(not(feature = "use-blas-lapack"))]
+            TypedTensor::Typef32(v1) => {
+                v1.svd().map(|[r1, r2, r3]| [TypedTensor::Typef32(r1),
+                                            TypedTensor::Typef32(r2),
+                                            TypedTensor::Typef32(r3),])
+            },
+	    #[cfg(not(feature = "use-blas-lapack"))]
+            TypedTensor::Typef64(v1) => {
+                v1.svd().map(|[r1, r2, r3]| [TypedTensor::Typef64(r1),
+                                            TypedTensor::Typef64(r2),
+                                            TypedTensor::Typef64(r3),])
+            },
+	    #[cfg(feature = "use-blas-lapack")]
+            TypedTensor::Typef32(v1) => {
+		let (u, s, v) = svd_f32(v1);
+		Some([TypedTensor::Typef32(u),
+                      TypedTensor::Typef32(s),
+                      TypedTensor::Typef32(v),])
+            },
+            #[cfg(feature = "use-blas-lapack")]
+            TypedTensor::Typef64(v1) => {
+		let (u, s, v) = svd_f64(v1);
+		Some([TypedTensor::Typef64(u),
+                      TypedTensor::Typef64(s),
+                      TypedTensor::Typef64(v),])
+            },
+            #[cfg(feature = "use-cuda")]
+            TypedTensor::Cudaf32(v1) => {
+                v1.svd().map(|[r1, r2, r3]| [TypedTensor::Cudaf32(r1),
+                                            TypedTensor::Cudaf32(r2),
+                                            TypedTensor::Cudaf32(r3),])
+            },
+            //_ => {panic!("should have same tensor type!");},
+        }
+    }
     typed_tensor_method_single_same_option_1_return!(inv);
     typed_tensor_method_single_tensor_return!(pinv);
     typed_tensor_method_single_tensor_return!(tr);
@@ -827,9 +932,9 @@ impl TypedTensor {
                   padding_mode: PaddingMode) -> TypedTensor {
         match (self, filter) {
             (TypedTensor::Typef32(v1), TypedTensor::Typef32(v2)) => {
-                #[cfg(not(feature = "use-blas"))]
+                #[cfg(not(feature = "use-blas-lapack"))]
                 return TypedTensor::Typef32(v1.conv2d(v2, stride, padding, dilation, padding_mode));
-                #[cfg(feature = "use-blas")]
+                #[cfg(feature = "use-blas-lapack")]
                 return TypedTensor::Typef32(gemm_conv_f32(v1, v2,
                                                           &[stride.0, stride.1],
                                                           &[padding.0, padding.1],
@@ -837,9 +942,9 @@ impl TypedTensor {
                                                           padding_mode));
             },
             (TypedTensor::Typef64(v1), TypedTensor::Typef64(v2)) => {
-                #[cfg(not(feature = "use-blas"))]
+                #[cfg(not(feature = "use-blas-lapack"))]
                 return TypedTensor::Typef64(v1.conv2d(v2, stride, padding, dilation, padding_mode));
-                #[cfg(feature = "use-blas")]
+                #[cfg(feature = "use-blas-lapack")]
                 return TypedTensor::Typef64(gemm_conv_f64(v1, v2,
                                                           &[stride.0, stride.1],
                                                           &[padding.0, padding.1],
@@ -881,8 +986,8 @@ impl fmt::Display for TypedTensor {
 impl fmt::Debug for TypedTensor {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            TypedTensor::Typef32(v) => write!(f, "typef32: {}", v),
-            TypedTensor::Typef64(v) => write!(f, "typef64: {}", v),
+            TypedTensor::Typef32(v) => write!(f, "typef32: {:?}", v),
+            TypedTensor::Typef64(v) => write!(f, "typef64: {:?}", v),
             //_ => panic!("Other case"),
         }
     }
